//	ERRORSUB.PRG -- Translate Clipper V5.0 Errors to S87 format
//
// The following #command definitons are included just for fun.
// With these included, you can say:
//	
//		ON ERROR DO ErrorSub SAVE OrigError
//
//	then later:
//
//		ON ERROR RESTORE OrigError
//
//	rather than the somewhat more cryptic
//
//		OrigError = ERRORBLOCK({|objError| ErrorSub(objError)})
//
// and
//
//		ERRORBLOCK(OrigError)
//
//	These are not critical to the function of ErrorSub, but they gave
// me a chance to play with #command and make my code more readable.
// Note:  I *tried* to recognize:
//
//		ON ERROR DO function SAVE xyzzy
//
//	and
//
//		ON ERROR DO (xyzzy)
//
// but I couldn't convince the preprocessor to behave differently based
// on the presence of parens.  Maybe I'll play with it more later, or
// maybe someone else will make it work.

#command ON ERROR [SAVE <previous>] RESTORE <CodeBlock>;
	=> [<previous> := ]ERRORBLOCK(<CodeBlock>)

#command ON ERROR [SAVE <previous>] DO <Handler>[(<arglist,...>)];
	=> [<previous> := ]ERRORBLOCK({|objError| <Handler>(objError[,<arglist>])})


//	ANYWAY....

//	The real point of this file is the following function which translates
// Clipper 5's error object to Clipper S87's haphazard errorsys calls.
// My mapping of genCodes to ????_error() calls is somewhat of a guess, but
// it works and it did smooth the transition from S87 to V5 for a project
// I was working on.  Please consider this as an interim measure only.  Once
// you get your system up you should rewrite your errorproc to take advantage
// of all the additonal information available with the new system.
//
//	To use this: compile this.
//
// Include an ERRORBLOCK({|objError|ErrorSub(ObjError)}) in your source
// program. (or an ON ERROR DO ErrorSub using the above #commands)
//
// Link your source program, this program AND your S87 ErrorSys module.							
//
//		Dale Wilson, Codewright Computer Services, Inc.
//		CID ID# 72155,1402
//
//	No warranty of any type is made for this code.


#include "error.ch"

FUNCTION ErrorSub(oErr)

	DO CASE
		CASE oErr:genCode = EG_ARG;
		.OR. oErr:genCode = EG_BOUND;
		.OR. oErr:genCode = EG_STROVERFLOW;
		.OR. oErr:genCode = EG_NUMOVERFLOW;
		.OR. oErr:genCode = EG_ZERODIV;
		.OR. oErr:genCode = EG_NUMERR
			RETURN expr_error(ProcName(3),ProcLine(3),oErr:description,oErr:operation,,"","")
							/* Note last 2 args SHOULD be oErr:arg[1] & [2]
								but it blows up whenever I reference :arg in ANY capacity! */

		CASE oErr:genCode = EG_SYNTAX;
		.OR. oErr:genCode = EG_COMPLEXITY;
		.OR. oErr:genCode = EG_MEM;
		.OR. oErr:genCode = EG_NOFUNC;
		.OR. oErr:genCode = EG_NOMETHOD;
		.OR. oErr:genCode = EG_NOVAR
			RETURN undef_error(ProcName(3),ProcLine(3),oErr:description,"",oErr:operation)
				/* Last arg should :arg[1], but... */
							
		CASE oErr:genCode = EG_OPEN
			RETURN open_error(ProcName(3),ProcLine(3),oErr:description,"",oErr:filename)

		CASE oErr:genCode = EG_NOALIAS;
		.OR. oErr:genCode = EG_CREATE;
		.OR. oErr:genCode = EG_CLOSE;
		.OR. oErr:genCode = EG_READ;
		.OR. oErr:genCode = EG_WRITE;
		.OR. oErr:genCode = EG_SHARED;
		.OR. oErr:genCode = EG_UNLOCKED;
		.OR. oErr:genCode = EG_READONLY
			RETURN db_error(ProcName(3),ProcLine(3),oErr:description)

		CASE oErr:genCode = EG_PRINT
			RETURN print_error(ProcName(3),ProcLine(3))

		CASE oErr:genCode = EG_UNSUPPORTED;
		.OR. oErr:genCode = EG_LIMIT;
		.OR. oErr:genCode = EG_CORRUPTION;
		.OR. oErr:genCode = EG_DATATYPE;
		.OR. oErr:genCode = EG_DATAWIDTH;
		.OR. oErr:genCode = EG_NOTABLE;
		.OR. oErr:genCode = EG_NOORDER
			RETURN misc_error(ProcName(3),ProcLine(3),oErr:description,oErr:operation)

		OTHERWISE
			RETURN misc_error(ProcName(3),ProcLine(3),oErr:description,"Undefined GenCode:  "+STR(oErr:gencode))
	END CASE

RETURN .F.

